var _interopRequireDefault=require("@babel/runtime/helpers/interopRequireDefault");Object.defineProperty(exports,"__esModule",{value:true});exports.getInstalledTemplateOptions=exports.getTemplateOptions=exports.applyTemplate=exports.checkIfTemplateInstalled=exports.addTemplate=exports.listTemplates=void 0;var _chalk=_interopRequireDefault(require("chalk"));
var _fs=_interopRequireDefault(require("fs"));
var _path=_interopRequireDefault(require("path"));
var _deepmerge=_interopRequireDefault(require("deepmerge"));
var _constants=require("../constants");
var _fileutils=require("../systemTools/fileutils");




var _common=require("../common");
var _pluginTools=require("../pluginTools");
var _prompt=require("../systemTools/prompt");
var _projectParser=require("../projectTools/projectParser");
var _configParser=require("../configTools/configParser");

var _templates=require("../../renativeTemplates/templates.json");










var listTemplates=function listTemplates(c){return new Promise(function(resolve,reject){
(0,_common.logTask)('listTemplates');
var opts=getTemplateOptions(c);
console.log(opts.asString);
resolve();
});};exports.listTemplates=listTemplates;

var addTemplate=function addTemplate(c,opts){return new Promise(function(resolve,reject){
(0,_common.logTask)('addTemplate');var
maxErrorLength=c.program.maxErrorLength;

c.files.project.config.templates=c.files.project.config.templates||{};

if(!c.files.project.config.templates[opts.selectedOption]){
c.files.project.config.templates[opts.selectedOption]={
version:'latest'};

}

_writeObjectSync(c,c.paths.project.config,c.files.project.config);


resolve();
});};exports.addTemplate=addTemplate;

var checkIfTemplateInstalled=function checkIfTemplateInstalled(c){return new Promise(function(resolve,reject){
(0,_common.logTask)('checkIfTemplateInstalled');
if(!c.buildConfig.templates){
(0,_common.logWarning)("Your "+_chalk.default.white(c.paths.project.config)+" does not contain "+_chalk.default.white('templates')+" object. ReNative will skip template generation");
resolve();
return;
}

for(var k in c.buildConfig.templates){
var t=k;
var obj=c.buildConfig.templates[k];
if(k.version&&k.version.startsWith('file:')){
t="../"+k.version.replace('file:','');
}
var templateFolder=_path.default.join(c.paths.project.nodeModulesDir,k);
if(!_fs.default.existsSync(templateFolder)){
(0,_common.logWarning)("Your "+_chalk.default.white(templateFolder)+" template is not installed. ReNative will install it for you");


c._requiresNpmInstall=true;
}
if(c.files.project.package.devDependencies){
c.files.project.package.devDependencies[k]=obj.version;
}
}
_writeObjectSync(c,c.paths.project.package,c.files.project.package);

resolve();
});};exports.checkIfTemplateInstalled=checkIfTemplateInstalled;

var applyTemplate=function applyTemplate(c,selectedTemplate){return new Promise(function(resolve,reject){
(0,_common.logTask)('applyTemplate');

if(!c.buildConfig.currentTemplate){
(0,_common.logWarning)('You don\'t have any current template selected');
var opts=getInstalledTemplateOptions(c);

(0,_prompt.askQuestion)("Pick which template to apply: \n"+opts.asString).
then(function(v){return opts.pick(v);}).
then(function(v){
(0,_prompt.finishQuestion)();
c.buildConfig.currentTemplate=opts.selectedOption;
c.files.project.config.currentTemplate=opts.selectedOption;
_writeObjectSync(c,c.paths.project.config,c.files.project.config);
return Promise.resolve();
}).
then(function(){return _preApplyTemplate(c,selectedTemplate);}).
then(function(){return resolve();}).
catch(function(e){return reject(e);});
}else{
_preApplyTemplate(c,selectedTemplate).
then(function(){return resolve();}).
catch(function(e){return reject(e);});
}
});};exports.applyTemplate=applyTemplate;

var _preApplyTemplate=function _preApplyTemplate(c,selectedTemplate){return new Promise(function(resolve,reject){
(0,_common.logTask)("_preApplyTemplate:"+selectedTemplate);
c.paths.projectTemplateFolder=_path.default.join(c.paths.project.nodeModulesDir,c.buildConfig.currentTemplate);

if(c.runtime.isWrapper){
_applyLocalTemplate(c,selectedTemplate).
then(function(){return resolve();}).
catch(function(e){return reject(e);});
}else{
_applyTemplate(c,selectedTemplate).
then(function(){return resolve();}).
catch(function(e){return reject(e);});
}
});};

var _applyLocalTemplate=function _applyLocalTemplate(c,selectedTemplate){return new Promise(function(resolve,reject){
(0,_common.logTask)("_applyLocalTemplate:"+selectedTemplate);
var currentTemplate=selectedTemplate||c.buildConfig.currentTemplate;
if(selectedTemplate){
(0,_common.logTask)("applyTemplate:"+selectedTemplate);

if(currentTemplate!==selectedTemplate){
(0,_common.logWarning)("Current template "+_chalk.default.red(currentTemplate)+" will be overriden by "+_chalk.default.green(selectedTemplate));
}

var dirsToRemove=[
_path.default.join(c.paths.project.projectConfig.dir),
_path.default.join(c.paths.project.srcDir),
_path.default.join(c.paths.project.appConfigsDir)];


var filesToRemove=c.buildConfig.defaults.supportedPlatforms.map(function(p){return _path.default.join(c.paths.project.dir,"index."+p+".js");});

(0,_fileutils.removeDirsSync)(dirsToRemove);

(0,_fileutils.removeFilesSync)(filesToRemove);

c.paths.projectTemplateFolder=_path.default.join(c.paths.project.nodeModulesDir,selectedTemplate);

_applyTemplate(c,selectedTemplate).
then(function(){return(0,_projectParser.configureEntryPoints)(c);}).
then(function(){return resolve();}).
catch(function(e){return reject(e);});

return resolve();
}
return resolve();
});};

var _applyTemplate=function _applyTemplate(c,selectedTemplate){return new Promise(function(resolve,reject){
(0,_common.logTask)("_applyTemplate:"+selectedTemplate+":"+c.paths.projectTemplateFolder);

var templateConfigPath=_path.default.join(c.paths.projectTemplateFolder,_constants.RENATIVE_CONFIG_TEMPLATE_NAME);

if(!_fs.default.existsSync(templateConfigPath)){
(0,_common.logWarning)("Template file "+_chalk.default.white(templateConfigPath)+" does not exist. check your "+_chalk.default.white(c.paths.projectTemplateFolder)+". skipping");
resolve();
return;
}

var templateAppConfigsFolder=_path.default.join(c.paths.projectTemplateFolder,'appConfigs');
var templateAppConfigFolder=_fs.default.readdirSync(templateAppConfigsFolder)[0];
var templateProjectConfigFolder=_path.default.join(c.paths.projectTemplateFolder,'projectConfig');
var currentTemplate=c.files.project.config.currentTemplate;
if(!currentTemplate){
currentTemplate=Object.keys(c.files.project.config.templates)[0];
c.runtime.requiresForcedTemplateApply=true;
}
var templateConfig=(0,_fileutils.readObjectSync)(templateConfigPath);


(0,_common.logTask)('configureProject:check src',_chalk.default.grey);
if(!_fs.default.existsSync(c.paths.project.srcDir)){
(0,_common.logInfo)("Looks like your src folder "+_chalk.default.white(c.paths.project.srcDir)+" is missing! Let's create one for you.");
(0,_fileutils.copyFolderContentsRecursiveSync)(_path.default.join(c.paths.projectTemplateFolder,'src'),c.paths.project.srcDir);
}


(0,_common.logTask)('configureProject:check appConfigs',_chalk.default.grey);

if(!_fs.default.existsSync(c.paths.project.appConfigsDir)){
(0,_common.logInfo)("Looks like your appConfig folder "+
_chalk.default.white(
c.paths.project.appConfigsDir)+" is missing! Let's create sample config for you.");




(0,_fileutils.copyFolderContentsRecursiveSync)(templateAppConfigsFolder,c.paths.project.appConfigsDir);

var appConfigIds=(0,_configParser.listAppConfigsFoldersSync)(c);


try{var _c$files$project,_c$files$project$defa;
appConfigIds.forEach(function(v){
var appConfigPath=_path.default.join(c.paths.project.appConfigsDir,v,_constants.RENATIVE_CONFIG_NAME);
var appConfig=(0,_fileutils.readObjectSync)(appConfigPath);
appConfig.common=appConfig.common||{};
if(!c.runtime.isWrapper){var _c$files$project$conf,_c$files$project$conf2,_c$files$project$conf3,_c$files$project$conf4;
appConfig.common.title=(_c$files$project$conf=c.files.project.config)==null?void 0:(_c$files$project$conf2=_c$files$project$conf.defaults)==null?void 0:_c$files$project$conf2.title;
appConfig.common.id=(_c$files$project$conf3=c.files.project.config)==null?void 0:(_c$files$project$conf4=_c$files$project$conf3.defaults)==null?void 0:_c$files$project$conf4.id;
}

_writeObjectSync(c,appConfigPath,appConfig);
});

var supPlats=(_c$files$project=c.files.project)==null?void 0:(_c$files$project$defa=_c$files$project.defaults)==null?void 0:_c$files$project$defa.supportedPlatforms;

if(supPlats){
for(var pk in appConfig.platforms){
if(!supPlats.includes(pk)){
delete appConfig.platforms[pk];
}
}
}
}catch(e){
(0,_common.logError)(e);
}
}


(0,_common.logTask)('configureProject:check projectConfigs',_chalk.default.grey);
if(!_fs.default.existsSync(c.paths.project.projectConfig.dir)){
(0,_common.logInfo)("Looks like your projectConfig folder "+
_chalk.default.white(c.paths.project.projectConfig.dir)+" is missing! Let's create one for you.");

(0,_fileutils.copyFolderContentsRecursiveSync)(templateProjectConfigFolder,c.paths.project.projectConfig.dir);
}


(0,_common.logTask)('configureProject:check renative.json',_chalk.default.grey);
if(!c.runtime.isWrapper){
if(selectedTemplate||c.runtime.requiresForcedTemplateApply||c.files.project.config.isNew){
(0,_common.logWarning)("Looks like your "+
c.paths.project.config+" need to be updated with "+templateConfigPath);

var mergedObj=(0,_fileutils.mergeObjects)(c,c.files.project.config,templateConfig,false,true);
mergedObj.currentTemplate=currentTemplate;
mergedObj.isNew=null;
delete mergedObj.isNew;
c.files.project.config=mergedObj;
_writeObjectSync(c,c.paths.project.config,mergedObj);
}
}else{
if(templateConfig.plugins.renative){
templateConfig.plugins.renative=(0,_pluginTools.getLocalRenativePlugin)();
}
_writeObjectSync(c,c.paths.project.configLocal,templateConfig);
}

(0,_configParser.setAppConfig)(c,c.runtime.appId);

resolve();
});};

var _writeObjectSync=function _writeObjectSync(c,p,s){
(0,_fileutils.writeObjectSync)(p,s);
(0,_configParser.generateBuildConfig)(c);
};

var getTemplateOptions=function getTemplateOptions(c){return(0,_prompt.generateOptions)(_templates.templates,false,null,function(i,obj,mapping,defaultVal){var _c$buildConfig$templa;
var exists=(_c$buildConfig$templa=c.buildConfig.templates)==null?void 0:_c$buildConfig$templa[defaultVal];
var installed=exists?_chalk.default.red(' (installed)'):'';
return"-["+_chalk.default.green(i+1)+"] "+_chalk.default.green(defaultVal)+installed+" \n";
});};exports.getTemplateOptions=getTemplateOptions;

var getInstalledTemplateOptions=function getInstalledTemplateOptions(c){
if(c.buildConfig.templates){
return(0,_prompt.generateOptions)(c.buildConfig.templates);
}
(0,_common.logError)('You don\'t have any local templates installed');
return[];
};exports.getInstalledTemplateOptions=getInstalledTemplateOptions;